jest.mock('ui/utils');

import {select} from 'd3-selection';

import * as utils from 'ui/utils';
import {appendXAxis, appendYAxis, appendSecondaryYAxis, appendAxes} from './axes';

describe('axes module', () => {
    let svg, layout;
    utils.wrap = jest.fn();

    beforeEach(() => {

        layout = {
            width: 550,
            height: 900,
            margin: {
                top: 10,
                bottom: 10,
                left: 25,
                right: 25
            }
        };
        svg = select('body').append('svg');

    });

    afterEach(() => {
        svg.remove();
    });

    describe('appendXAxis', () => {
        let xAxis;
        beforeEach(() => {
            xAxis = jest.fn();

        });
        it('should render the xAxis', () =>  {
            appendXAxis(svg, {xAxis, layout});

            expect(svg.selectAll('.x-axis').size()).toBe(1);
            expect(xAxis).toHaveBeenCalled();
        });
    });

    describe('appendYAxis', () => {
        let yAxis;
        beforeEach(() => {
            yAxis = jest.fn();
        });

        it('should render the yAxis', () =>  {
            appendYAxis(svg, {yAxis, layout, yTitle: 'The Y Axis Title'});

            expect(svg.selectAll('.y-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis-label').size()).toBe(1);
            expect(svg.select('.y-axis-label').text()).toEqual('The Y Axis Title');
            expect(yAxis).toHaveBeenCalled();
        });
    });

    describe('appendSecondaryYAxis', () => {
        let yAxis;
        beforeEach(() => {
            yAxis = jest.fn();
        });

        it('should render the secondaryYAxis', () =>  {
            appendSecondaryYAxis(svg, {yAxis, layout, yTitle: 'The Secondary Y Axis Title'});

            expect(svg.selectAll('.secondary-y-axis').size()).toBe(1);
            expect(svg.select('.secondary-y-axis').selectAll('.y-axis-label').size()).toBe(1);
            expect(svg.select('.secondary-y-axis').select('.y-axis-label').text()).toEqual('The Secondary Y Axis Title');
            expect(yAxis).toHaveBeenCalled();
        });
    });

    describe('appendAxes', () => {
        let xAxis, primaryYAxis, secondaryYAxis, secondaryParameter;
        beforeEach(() => {
            xAxis = jest.fn();
            primaryYAxis = jest.fn();
            secondaryYAxis = jest.fn();
            secondaryParameter = jest.fn();
        });

        it('should render all three axes if a secondaryYAxis is defined', () => {
            secondaryYAxis = jest.fn();
            appendAxes(svg, {xAxis, primaryYAxis, secondaryYAxis, layout, primaryYTitle: 'ft', secondaryYTitle: 'ft/3', secondaryParameter});

            expect(svg.selectAll('.x-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis').size()).toBe(1);
            expect(svg.selectAll('.secondary-y-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis-label').size()).toBe(2);
            expect(svg.select('.y-axis .y-axis-label').html()).toBe('ft');
            expect(svg.select('.secondary-y-axis .y-axis-label').html()).toBe('ft/3');
        });

        it('expects that secondary axis is removed if a secondary parameter is no longer defined', () => {
            secondaryYAxis = jest.fn();
            appendAxes(svg, {xAxis, primaryYAxis, secondaryYAxis, layout, primaryYTitle: 'ft', secondaryYTitle: 'ft/3', secondaryParameter});

            expect(svg.selectAll('.x-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis').size()).toBe(1);
            expect(svg.selectAll('.secondary-y-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis-label').size()).toBe(2);
            expect(svg.select('.y-axis .y-axis-label').html()).toBe('ft');
            expect(svg.select('.secondary-y-axis .y-axis-label').html()).toBe('ft/3');

            appendAxes(svg, {xAxis, primaryYAxis, layout, primaryYTitle: 'ft'});
            expect(svg.selectAll('.x-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis').size()).toBe(1);
            expect(svg.selectAll('.secondary-y-axis').size()).toBe(0);
        });

        it('should render only one y-axis if a secondaryYAxis is not defined', () => {
            appendAxes(svg, {xAxis, primaryYAxis, secondaryYAxis, layout, primaryYTitle: 'ft', secondaryYTitle: '', secondaryParameter: null});

            expect(svg.selectAll('.x-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis').size()).toBe(1);
            expect(svg.selectAll('.y-axis-label').size()).toBe(1);
            expect(svg.selectAll('.secondary-y-axis').size()).toBe(0);
        });
    });
});