
import config from 'ui/config';

import {hasMeasuredFahrenheitParameter, isCalculatedTemperature, getConvertedTemperatureParameter, getPeriodOfRecord
} from './parameter-code-utils';

describe('monitoring-location/parameter-code-utils', () => {

    describe('hasMeasuredFahrenheitParameter', () => {
        const allParameterCodes = ['72019', '00020', '00021', '00010', '45589'];

        it('Return false if the code does not have a fahrenheit counterpart', () => {
            expect(hasMeasuredFahrenheitParameter('72019', allParameterCodes)).toBe(false);
        });

        it('Return true if the measured fahrenheit parameter is in the list', () => {
            expect(hasMeasuredFahrenheitParameter('00020', allParameterCodes)).toBe(true);
        });

        it('Return false if the measured fahrenheit parameter is not in the list', () => {
            expect(hasMeasuredFahrenheitParameter('00010', allParameterCodes)).toBe(false);
            expect(hasMeasuredFahrenheitParameter('45589', allParameterCodes)).toBe(false);

        });
    });

    describe('isCalculatedTemperature', () => {
        it('Expects to return true if this a calculated temperature parameter code', () => {
            expect(isCalculatedTemperature('00010F')).toBe(true);
        });

        it('Expects to return false if this is not a calculated temperature parameter code', () => {
            expect(isCalculatedTemperature('00011')).toBe(false);
        });

        it('Expects to return false if the parameter code is null', () => {
            expect(isCalculatedTemperature(null)).toBe(false);
        });
    });

    describe('getConvertedTemperatureParameter', () => {
        it('Expects to convert the temperature parameter object to a calculated parameter object', () => {
           expect(getConvertedTemperatureParameter({
               parameterCode: '00010',
               name: 'Temperature, water, C',
               description: 'Temperature, water, degrees Celsius',
               unit: 'deg C',
               hasIVData: true
           })).toEqual({
               parameterCode: '00010F',
               name: 'Temperature, water, F (calculated)',
               description: 'Temperature, water, degrees Fahrenheit (calculated)',
               unit: 'deg F',
               hasIVData: true
           });
        });
    });

    describe('getPeriodOfRecord', () => {
        const TEST_IV_PERIOD_OF_RECORD = {
            '00010': {
                begin_date: '2020-01-01',
                end_date: '2021-10-03'
            },
            '00060': {
                begin_date: '2019-01-01',
                end_date: '2021-10-01'
            },
            '72019': {
                begin_date: '2019-11-11',
                end_date: '2021-11-03'
            }
        };

        const TEST_GW_PERIOD_OF_RECORD = {
            '72019': {
                begin_date: '2020-01-01',
                end_date: '2021-11-04'
            },
            '62611': {
                begin_date: '2019-01-15',
                end_date: '2021-12-04'
            }
        };

        it('expects null strings if the config does not contain the config constants', () => {
            config.ivPeriodOfRecord = undefined;
            config.gwPeriodOfRecord = undefined;
            expect(getPeriodOfRecord('00065')).toEqual({
                begin_date: '',
                end_date: ''
            });
        });
        it('Expects that period of record is retrieved from IV if only IV data is available', () => {
            config.ivPeriodOfRecord = TEST_IV_PERIOD_OF_RECORD;
            config.gwPeriodOfRecord = undefined;
            expect(getPeriodOfRecord('00060')).toEqual({
                'begin_date': '2019-01-01',
                'end_date': '2021-10-01'
            });
            expect(getPeriodOfRecord('00065')).toEqual({
                'begin_date': '',
                'end_date': ''
            });
        });

        it('Expects that period of record is retrieved GW if only GW data is available', () => {
            config.ivPeriodOfRecord = undefined;
            config.gwPeriodOfRecord = TEST_GW_PERIOD_OF_RECORD;

            expect(getPeriodOfRecord('72019')).toEqual({
                'begin_date': '2020-01-01',
                'end_date': '2021-11-04'
            });
            expect(getPeriodOfRecord('00060')).toEqual({
                'begin_date': '',
                'end_date': ''
            });
        });

        it('Expects that period of record is correctly retrieved when both IV Data and GW data is available', () => {
            config.ivPeriodOfRecord = TEST_IV_PERIOD_OF_RECORD;
            config.gwPeriodOfRecord = TEST_GW_PERIOD_OF_RECORD;

            expect(getPeriodOfRecord('00060')).toEqual({
                begin_date: '2019-01-01',
                end_date: '2021-10-01'
            });
            expect(getPeriodOfRecord('72019')).toEqual({
                begin_date: '2019-11-11',
                end_date: '2021-11-04'
            });
            expect(getPeriodOfRecord('62611')).toEqual({
                begin_date: '2019-01-15',
                end_date: '2021-12-04'
            });
        });

        it('Expect calculcated temperature parameter code to be handled correctly', () => {
            config.ivPeriodOfRecord = TEST_IV_PERIOD_OF_RECORD;
            config.gwPeriodOfRecord = TEST_GW_PERIOD_OF_RECORD;

            expect(getPeriodOfRecord('00010')).toEqual({
                begin_date: '2020-01-01',
                end_date: '2021-10-03'
            });
            expect(getPeriodOfRecord('00010F')).toEqual({
                begin_date: '2020-01-01',
                end_date: '2021-10-03'
            });
        });
    });
});
