import {getAllGroundwaterLevels, getSelectedGroundwaterLevels} from './groundwater-level-field-visits-selector';

describe('monitoring-location/selectors/groundwater-level-field-visits-selector', () => {
   const TEST_GW_LEVELS_DATA = [{
      observedPropertyName: 'Groundwater level above NGVD 1929, feet',
      USGSParameterCode: '62610',
      unitOfMeasureName: 'ft',
      observations: [{
         phenomenonTime: '2021-11-15T21:29',
         phenomenonTimeAccuracy: 'minute',
         result: '1315.49',
         resultAccuracy: '0.01',
         approvals: ['Approved'],
         qualifiers: ['Static'],
         epochTime: 1637011740000
      }, {
         phenomenonTime: '2021-11-16T16:03',
         phenomenonTimeAccuracy: 'minute',
         result: '1315.45',
         resultAccuracy: '0.01',
         approvals: ['Approved'],
         qualifiers: ['Static'],
         epochTime: 1637078580000
      }, {
         phenomenonTime: '2022-01-27T19:53',
         phenomenonTimeAccuracy: 'minute',
         result: '1314.33',
         resultAccuracy: '0.01',
         approvals: ['Provisional'],
         qualifiers: ['Static'],
         epochTime: 1643313180000
      }]
   }, {
      observedPropertyName: 'Depth to water level, feet below land surface',
      USGSParameterCode: '72019',
      unitOfMeasureName: 'ft',
      observations: [{
         phenomenonTime: '2021-11-15T21:29',
         phenomenonTimeAccuracy: 'minute',
         result: '29.40',
         resultAccuracy: '0.01',
         approvals: ['Approved'],
         qualifiers: ['Static'],
         epochTime: 1637011740000
      }, {
         phenomenonTime: '2021-11-16T16:03',
         phenomenonTimeAccuracy: 'minute',
         result: '29.44',
         resultAccuracy: '0.01',
         approvals: ['Approved'],
         qualifiers: ['Static'],
         epochTime: 1637078580000
      }, {
         phenomenonTime: '2022-01-27T19:53',
         phenomenonTimeAccuracy: 'minute',
         result: '30.56',
         resultAccuracy: '0.01',
         approvals: ['Provisional'],
         qualifiers: ['Static'],
         epochTime: 1643313180000
      }]
   }];

   describe('getAllGroundwterLevels', () => {
      it('Expects to return the groundwater levels saved', () => {
         expect(getAllGroundwaterLevels({
            groundwaterLevelData: {
               all: TEST_GW_LEVELS_DATA
            }
         })).toEqual(TEST_GW_LEVELS_DATA);
      });
   });

   describe('getSelectedGroundwaterLevels', () => {

      it('Expects null if the selected parameter code does not have any gw data', () => {
         expect(getSelectedGroundwaterLevels({
            hydrographData: {
               currentTimeRange: {
                  start: 1643313180000,
                  end: 1653313180000
               }
            },
            hydrographState: {
               selectedParameterCode: '00065'
            },
            groundwaterLevelData: {
               all: TEST_GW_LEVELS_DATA
            }
         })).toBeNull();
      });

      it('Expects null if the currentTimeRange is not yet set', () => {
         expect(getSelectedGroundwaterLevels({
            hydrographData: {
            },
            hydrographState: {
               selectedParameterCode: '72019'
            },
            groundwaterLevelData: {
               all: TEST_GW_LEVELS_DATA
            }
         })).toBeNull();
      });

      it('Expects the selected observations to be returned when the selectedParameterCode is groundwater data', () => {
         const result = getSelectedGroundwaterLevels({
            hydrographData: {
               currentTimeRange: {
                  start: 1639313180000,
                  end: 1653313180000
               }
            },
            hydrographState: {
               selectedParameterCode: '72019'
            },
            groundwaterLevelData: {
               all: TEST_GW_LEVELS_DATA
            }
         });

         expect(result.parameter).toEqual({
            parameterCode: '72019',
            name: 'Depth to water level, feet below land surface',
            description: 'Depth to water level, feet below land surface',
            unit: 'ft'
         });
         expect(result.values).toHaveLength(1);
         expect(result.values[0]).toEqual({
            phenomenonTime: '2022-01-27T19:53',
            phenomenonTimeAccuracy: 'minute',
            result: '30.56',
            resultAccuracy: '0.01',
            approvals: ['Provisional'],
            qualifiers: ['Static'],
            epochTime: 1643313180000
         });
      });

      it('Expects an empty values array if no groundwater levels in the time range', () => {
  const result = getSelectedGroundwaterLevels({
            hydrographData: {
               currentTimeRange: {
                  start: 1539313180000,
                  end: 1553313180000
               }
            },
            hydrographState: {
               selectedParameterCode: '72019'
            },
            groundwaterLevelData: {
               all: TEST_GW_LEVELS_DATA
            }
         });

         expect(result.parameter).toEqual({
            parameterCode: '72019',
            name: 'Depth to water level, feet below land surface',
            description: 'Depth to water level, feet below land surface',
            unit: 'ft'
         });
         expect(result.values).toHaveLength(0);
      });
   });
});
