import config from 'ui/config';

import {getThresholds, getTimeRange, getIVData, getStatisticsData, getIVParameter,
    getIVMethods, getPrimaryMedianStatisticsData, getPrimaryMedianStatisticsValueRange
} from './hydrograph-data-selector';

describe('monitoring-location/selectors/hydrograph-data-selectors', () => {
    config.locationTimeZone = 'America/Chicago';

    const TEST_PRIMARY_IV_DATA = {
        parameter: {
            parameterCode: '00060',
            name: 'Streamflow ft3/s',
            description: 'Discharge, cubic feet per second',
            unit: 'ft3/s'
        },
        values: {
            '15776': {
                points: [
                    {value: '18.2', dateTime: 1613421000000},
                    {value: '20.2', dateTime: 1613421900000},
                    {value: '19.9', dateTime: 1613422800000}],
                method: {methodID: '15776'}
            }
        }
    };

    const TEST_SECONDARY_IV_DATA = {
        parameter: {
            parameterCode: '00065',
            name: 'Gage height, feet',
            description: 'Gage height, feet',
            unit: 'ft'
        },
        values: {
            '43043': {
                points: [
                    {value: '3.3', dateTime: 1613421000000},
                    {value: '3.6', dateTime: 1613421900000},
                    {value: '3.9', dateTime: 1613422800000}],
                method: {methodID: '43043'}
            }
        }
    };

    const TEST_COMPARE_IV_DATA = {
        parameter: {
            parameterCode: '00060',
            name: 'Streamflow ft3/s',
            description: 'Discharge, cubic feet per second',
            unit: 'ft3/s'
        },
        values: {
            '15776': {
                points: [
                    {value: '15.2', dateTime: 1581885000000},
                    {value: '13.2', dateTime: 1581885900000},
                    {value: '12.9', dateTime: 1581886800000}],
                method: {methodID: '15776'}
            }
        }
    };

    const TEST_STATS_DATA = {
        '153885': [
            {month_nu: 2, day_nu: 1, p50_va: 16, ts_id: '153885', loc_web_ds: 'Method1', begin_yr: '2011', end_yr: '2020', max_va_yr: '2020',
                max_va: '273', min_va_yr: '2006', min_va: '55.5', mean_va: '153', p05_va: '', p10_va: '61', p20_va: '88', p25_va: '100',
                p75_va: '224', p80_va: '264', p90_va: '271', p95_va: ''},
            {month_nu: 2, day_nu: 2, p50_va: 16.2, ts_id: '153885', loc_web_ds: 'Method1', begin_yr: '2011', end_yr: '2020', max_va_yr: '2020',
                max_va: '273', min_va_yr: '2006', min_va: '55.5', mean_va: '153', p05_va: '', p10_va: '61', p20_va: '88', p25_va: '100',
                p75_va: '224', p80_va: '264', p90_va: '271', p95_va: ''},
            {month_nu: 2, day_nu: 3, p50_va: 15.9, ts_id: '153885', loc_web_ds: 'Method1', begin_yr: '2011', end_yr: '2020', max_va_yr: '2020',
                max_va: '273', min_va_yr: '2006', min_va: '55.5', mean_va: '153', p05_va: '', p10_va: '61', p20_va: '88', p25_va: '100',
                p75_va: '224', p80_va: '264', p90_va: '271', p95_va: ''},
            {month_nu: 2, day_nu: 4, p50_va: 16.3, ts_id: '153885', loc_web_ds: 'Method1', begin_yr: '2011', end_yr: '2020', max_va_yr: '2020',
                max_va: '273', min_va_yr: '2006', min_va: '55.5', mean_va: '153', p05_va: '', p10_va: '61', p20_va: '88', p25_va: '100',
                p75_va: '224', p80_va: '264', p90_va: '271', p95_va: ''},
            {month_nu: 2, day_nu: 4, p50_va: 16.4, ts_id: '153885', loc_web_ds: 'Method1', begin_yr: '2011', end_yr: '2020', max_va_yr: '2020',
                max_va: '273', min_va_yr: '2006', min_va: '55.5', mean_va: '153', p05_va: '', p10_va: '61', p20_va: '88', p25_va: '100',
                p75_va: '224', p80_va: '264', p90_va: '271', p95_va: ''},
            {month_nu: 2, day_nu: 4, p50_va: 16.5, ts_id: '153885', loc_web_ds: 'Method1', begin_yr: '2011', end_yr: '2020', max_va_yr: '2020',
            max_va: '273', min_va_yr: '2006', min_va: '55.5', mean_va: '153', p05_va: '', p10_va: '61', p20_va: '88', p25_va: '100',
            p75_va: '224', p80_va: '264', p90_va: '271', p95_va: ''}
            ]
    };

    const TEST_THRESHOLDS = {
        operating_limits: [{
            parameterCode: '00065',
            methodDescription: '',
            thresholdDetails: [{
                name: 'Operational Limit (Maximum)',
                referenceCode: 'Operational limit - high-Public',
                referenceValue: 30
            }, {
                name: 'Below opereration limit',
                referenceCode: 'Operational limit - low-Public',
                referenceValue: 0
            }]
        }]
    };

    describe('getThresholds', () => {
       it('Returns null if thresholds are not defined', () => {
           expect(getThresholds({
               hydrographData: {}
           })).toBeNull();
       });

       it('Returns thresholds if defined', () => {
           expect(getThresholds({
               hydrographData: {
                   thresholds: TEST_THRESHOLDS
               }
           })).toEqual(TEST_THRESHOLDS);
       });
    });

    describe('getTimeRange', () => {
        it('returns null if time range is not defined', () => {
            expect(getTimeRange('current')({
                hydrographData: {}
            })).toBeNull();
        });

        it('returns the stored time range for the timeRangeKind', () => {
            expect(getTimeRange('current')({
                hydrographData: {
                    currentTimeRange: {
                        start: 1613413736967,
                        end: 1614018536967
                    },
                    prioryearTimeRange: {
                        start: 1581877736967,
                        end: 1582482536967
                    }
                }
            })).toEqual({
                start: 1613413736967,
                end: 1614018536967
            });
        });
    });

    describe('getIVData', () => {
        it('returns null if no iv data of data kind is store', () => {
            expect(getIVData('primary')({
                hydrographData: {}
            })).toBeNull();
        });

        it('Returns the selected IV Data', () => {
            expect(getIVData('primary')({
               hydrographData: {
                   primaryIVData: TEST_PRIMARY_IV_DATA,
                   secondaryIVData: TEST_SECONDARY_IV_DATA,
                   compareIVData: TEST_COMPARE_IV_DATA
               }
            })).toEqual(TEST_PRIMARY_IV_DATA);
            expect(getIVData('compare')({
               hydrographData: {
                   primaryIVData: TEST_PRIMARY_IV_DATA,
                   secondaryIVData: TEST_SECONDARY_IV_DATA,
                   compareIVData: TEST_COMPARE_IV_DATA
               }
            })).toEqual(TEST_COMPARE_IV_DATA);
            expect(getIVData('secondary')({
                hydrographData: {
                    primaryIVData: TEST_PRIMARY_IV_DATA,
                    secondaryIVData: TEST_SECONDARY_IV_DATA,
                    compareIVData: TEST_COMPARE_IV_DATA
                }
            })).toEqual(TEST_SECONDARY_IV_DATA);
        });
    });

    describe('getStatisticsData', () => {
        it('Returns null if no stats data', () => {
            expect(getStatisticsData({
                hydrographData: {}
            })).toBeNull();
        });

        it('Returns the stats data', () => {
            expect(getStatisticsData({
                hydrographData: {
                    statisticsData: TEST_STATS_DATA
                }
            })).toEqual(TEST_STATS_DATA);
        });
    });

    describe('getIVParameter', () => {
        it('Returns null if no IV levels are defined', () => {
            expect(getIVParameter('primary') ({
                hydrographData: {}
            })).toBeNull();
        });

        it('Returns the parameter from IV if defined', () => {
            expect(getIVParameter('primary') ({
                hydrographData: {
                    primaryIVData: TEST_PRIMARY_IV_DATA
                }
            })).toEqual(TEST_PRIMARY_IV_DATA.parameter);
        });
    });

    describe('getIVMethods', () => {
        it('Returns empty array if no IV data', () => {
            expect(getIVMethods('primary')({
                hydrographData: {}
            })).toEqual([]);
        });

        it('Returns the methods array if primary IV Data', () => {
            expect(getIVMethods('primary')({
                hydrographData: {
                    primaryIVData: TEST_PRIMARY_IV_DATA
                }
            })).toEqual([{methodID: '15776'}]);
        });
    });

    describe('getPrimaryMedianStatisticsData', () => {
        it('Returns null if no median statistics data', () => {
            expect(getPrimaryMedianStatisticsData({
                hydrographData: {}
            })).toBeNull();
        });

        it('Returns median statistics data for current time range', () => {
            expect(getPrimaryMedianStatisticsData({
                hydrographData: {
                    currentTimeRange: {
                        start: 1612280374000,
                        end: 1612539574000
                    },
                    statisticsData: TEST_STATS_DATA
                }
            })).toEqual({
                '153885': {
                    values: [
                        {point: 16.2, dateTime: 1612280374000},
                        {point: 15.9, dateTime: 1612332000000},
                        {point: 16.3, dateTime: 1612418400000},
                        {point: 16.3, dateTime: 1612539574000}
                    ],
                    description: 'Method1',
                    beginYear: '2011',
                    endYear: '2020'
                }
            });
        });
    });

    describe('getPrimaryMedianStatisticsValueRange', () => {
        it('Return null if no statistics data', () => {
            expect(getPrimaryMedianStatisticsValueRange({
                hydrographData: {}
            })).toBeNull();
        });

        it('Returns median statistics value range for current time range', () => {
            expect(getPrimaryMedianStatisticsValueRange({
                hydrographData: {
                    currentTimeRange: {
                        start: 1612280374000,
                        end: 1612539574000
                    },
                    statisticsData: TEST_STATS_DATA
                }
            })).toEqual([15.9, 16.3]);
        });
    });

});