
import {DateTime} from 'luxon';
import {createSelector} from 'reselect';

import config from 'ui/config';

import {getPeriodOfRecord} from 'ml/parameter-code-utils';

/*
 * The following selector functions return a function which returns the selected data.
 */
export const isCompareIVDataVisible = state => state.hydrographState.showCompareIVData || false;
export const isMedianDataVisible = state => state.hydrographState.showMedianData || false;
export const getSelectedTimeSpan = state => state.hydrographState.selectedTimeSpan || null;
export const getSelectedParameterCode = state => state.hydrographState.selectedParameterCode || null;
export const getSelectedSecondaryParameterCode = state => state.hydrographState.selectedSecondaryParameterCode || null;
export const getSelectedPrimaryIVMethodID = state => state.hydrographState.selectedIVMethodID || null;
export const getSelectedSecondaryIVMethodID = state => state.hydrographState.selectedSecondaryIVMethodID || null;
export const getGraphCursorOffset = state => state.hydrographState.graphCursorOffset || null;
export const getGraphBrushOffset = state => state.hydrographState.graphBrushOffset || null;

/*
 * Returns a selector function that returns an Object that can be used when calling
 * hydrographDataReducer action that retrieves data for the hydrograph. Note that is the timeSpan
 * represents a date range, the returned startTime is at midnight on the start date and the
 * endTime is 11:59:59 of the end date
 * @return {Function}
 */
export const getInputsForRetrieval = createSelector(
    getSelectedParameterCode,
    getSelectedSecondaryParameterCode,
    getSelectedTimeSpan,
    isCompareIVDataVisible,
    (parameterCode, secondaryParameterCode, selectedTimeSpan, loadCompare) => {
        const timeSpanIsDuration = typeof selectedTimeSpan === 'string';
        const period = timeSpanIsDuration && selectedTimeSpan.endsWith('D') ? selectedTimeSpan : null;
        let startTime = null;
        let endTime = null;
        if (!period) {
            if (timeSpanIsDuration) {
                const now = DateTime.now().setZone(config.locationTimeZone);
                if (selectedTimeSpan.endsWith('Y')) {
                    const years = parseInt(selectedTimeSpan.slice(1, -1));
                    startTime = now.minus({years: years}).startOf('day').toISO();
                    endTime = now.toISO();
                } else {
                    // Assuming that the time span is the period of record for the parameter code
                    const periodOfRecord = getPeriodOfRecord(parameterCode);
                    startTime = periodOfRecord.begin_date ?
                        DateTime.fromISO(periodOfRecord.begin_date, {zone: config.locationTimeZone}).toISO()
                        : now.minus({days: 1}).startOf('day').toISO();
                    endTime = now.toISO();
                }
            } else {
                startTime = DateTime.fromISO(selectedTimeSpan.start, {zone: config.locationTimeZone}).toISO();
                endTime = DateTime.fromISO(selectedTimeSpan.end, {zone: config.locationTimeZone}).endOf('day').toISO();
            }
        }

        return {
            parameterCode,
            secondaryParameterCode,
            period,
            startTime,
            endTime,
            loadCompare
        };
    }
);
