import {applyMiddleware, combineReducers, createStore} from 'redux';
import {default as thunk} from 'redux-thunk';

import {MOCK_GW_FIELD_VISITS, MOCK_JSON_ONE_DATASTREAM} from 'ui/mock-gw-field-visits';

import * as gwLevelsService from 'ui/web-services/groundwater-levels';

import config from 'ui/config';

import {groundwaterLevelDataReducer, retrieveGroundwaterLevelData} from './groundwater-level-field-visits';

describe('monitoring-location/web-services/groundwater-level-field-visits', () => {
    let store;

    config.locationTimeZone = 'America/Chicago';

    beforeEach(() => {
        store = createStore(
            combineReducers({
                groundwaterLevelData: groundwaterLevelDataReducer
            }),
            {
                groundwaterLevelData: {
                    all: []
                }
            },
            applyMiddleware(thunk)
        );
    });

    describe('retrieveGroundwaterLevelData', () => {

            it('Expects that the groundwater level service is called and the expected datastreams are put into the store', async() => {
                gwLevelsService.fetchGroundwaterLevels =
                    jest.fn().mockReturnValue(Promise.resolve(JSON.parse(MOCK_GW_FIELD_VISITS)));
                const augmentedDatastream =  await store.dispatch(retrieveGroundwaterLevelData('413703087230501', 'USGS'));

                expect(gwLevelsService.fetchGroundwaterLevels).toHaveBeenCalledWith({
                    siteno: '413703087230501',
                    agencyCd: 'USGS'
                });
                expect(augmentedDatastream).toHaveLength(3);
                expect(augmentedDatastream[0].observations).toHaveLength(24);
                const state = store.getState().groundwaterLevelData.all;
                expect(state).toHaveLength(3);
                expect(state[0].observations).toHaveLength(24);
            });

            it('Expects the epochTime to be added with the appropriate time zone and precision', async() => {
                gwLevelsService.fetchGroundwaterLevels =
                    jest.fn().mockReturnValue(Promise.resolve(MOCK_JSON_ONE_DATASTREAM));
                await store.dispatch(retrieveGroundwaterLevelData('413703087230501', 'USGS'));

                const observations = store.getState().groundwaterLevelData.all[0].observations;
                const minuteObservation = observations.find(observation => observation.phenomenonTimeAccuracy === 'minute');
                expect(minuteObservation.phenomenonTime).toEqual('2021-04-23T16:34');
                expect(minuteObservation.epochTime).toEqual(1619195640000);

                const hourObservation = observations.find(observation => observation.phenomenonTimeAccuracy === 'hour');
                expect(hourObservation.phenomenonTime).toEqual('2019-11-01T17');
                expect(hourObservation.epochTime).toEqual(1572627600000);

                const dayObservation = observations.find(observation => observation.phenomenonTimeAccuracy === 'day');
                expect(dayObservation.phenomenonTime).toEqual('2020-01-29');
                expect(dayObservation.epochTime).toEqual(1580277600000);

                const monthObservation = observations.find(observation => observation.phenomenonTimeAccuracy === 'month');
                expect(monthObservation.phenomenonTime).toEqual('2021-08');
                expect(monthObservation.epochTime).toEqual(1627794000000);

                const yearObservation = observations.find(observation => observation.phenomenonTimeAccuracy === 'year');
                expect(yearObservation.phenomenonTime).toEqual('2021');
                expect(yearObservation.epochTime).toEqual(1609480800000);
            });

            it('Expects that the observations are sorted in ascending epoch order', async() => {
                gwLevelsService.fetchGroundwaterLevels =
                    jest.fn().mockReturnValue(Promise.resolve(MOCK_JSON_ONE_DATASTREAM));
                await store.dispatch(retrieveGroundwaterLevelData('413703087230501', 'USGS'));

                const observations = store.getState().groundwaterLevelData.all[0].observations;
                const epochTimes = observations.map(observation => observation.epochTime);
                expect(epochTimes).toEqual([1572627600000, 1580277600000, 1609480800000, 1619195640000, 1627794000000]);
            });

            it('Expects if the fetch returns an empty object the store will contain an empty groundwater level visit array', async() => {
                gwLevelsService.fetchGroundwaterLevels = jest.fn().mockReturnValue(Promise.resolve({}));
                await store.dispatch(retrieveGroundwaterLevelData('413703087230501', 'USGS'));

                const state = store.getState().groundwaterLevelData.all;
                expect(state).toHaveLength(0);
            });
        });

});
