import config from 'ui/config';

const FLOOD_SITES_ENDPOINT = `${config.FIM_GIS_ENDPOINT}sites/MapServer/`;
const FLOOD_EXTENTS_ENDPOINT = `${config.FIM_GIS_ENDPOINT}floodExtents/MapServer/`;

/*
 * Determine if a site has public FIM data
 * @param {String} siteno
 * @return {Promise} resolves to a boolean, true if public, false otherwise
 */
export const fetchFIMPublicStatus = async function(siteno) {
    const query = `where=SITE_NO='${siteno}'&outFields=PUBLIC,SITE_NO&f=json`;
    const url = `${FLOOD_SITES_ENDPOINT}0/query?${encodeURI(query)}`;
    try {
        const response = await fetch(url, {
            method: 'GET'
        });
        if (response.status === 200) {
            const respJSON = await response.json();
            return respJSON.features[0].attributes.Public > 0;
        } else {
            console.error(`Received bad status, ${response.status} from ${url}`);
            return false;
        }
    } catch (error) {
        console.error(`Failed fetch for ${url}`);
        return false;
    }
};

/*
 * Retrieve flood features if any for siteno
 * @param {String} siteno
 * @return {Promise} resolves to an array of features for the site
 */
export const fetchFloodFeatures = async function(siteno) {
    const query = `where=USGSID = '${siteno}'&outFields=USGSID,STAGE&returnGeometry=false&f=json`;
    const url = `${FLOOD_EXTENTS_ENDPOINT}0/query?${encodeURI(query)}`;
    try {
        const response = await fetch(url, {
            method: 'GET'
        });
        if (response.status === 200) {
            const respJSON = await response.json();
            return respJSON.features ? respJSON.features : [];
        } else {
            console.error(`Received bad status, ${response.status} for ${url}`);
            return [];
        }
    } catch (error) {
        console.error(`Failed fetch for ${url}`);
        return [];
    }
};

/*
 * Retrieve the extent of the flood information for siteno
 * @param {String} siteno
 * @return {Promise} resolves to the extent Object or the empty object if any errors
 */
export const fetchFloodExtent = async function(siteno) {
    const query = `where=USGSID = '${siteno}'&returnExtentOnly=true&outSR=4326&f=json`;
    const url = `${FLOOD_EXTENTS_ENDPOINT}0/query?${encodeURI(query)}`;
    try {
        const response = await fetch(url, {
            method: 'GET'
        });
        if (response.status === 200) {
            return await response.json();
        } else {
            console.error(`Received bad status, ${response.status} for ${url}`);
            return {};
        }
    } catch (error) {
        console.error(`Failed fetch for ${url}`);
        return {};
    }
};
