import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';

import {MOCK_GW_FIELD_VISITS} from 'ui/mock-gw-field-visits';

import {getGroundwaterServiceURL, fetchGroundwaterLevels} from './groundwater-levels';

describe('web-services/groundwater-levels', () => {
    let restoreConsole;

    beforeEach(() => {
        restoreConsole = mockConsole();
        enableFetchMocks();
    });

    afterEach(() => {
        restoreConsole();
        disableFetchMocks();
    });

    describe('getGroundwaterServiceURL', () => {
        it('Expects site no, agencyCd and format parameters', () => {
            const result = getGroundwaterServiceURL({
                siteno: '11112222',
                agencyCd: 'USGS',
                format: 'json'
            });
            expect(result).toContain('featureId=USGS-11112222');
            expect(result).toContain('f=json');
        });
    });

    describe('fetchGroundwaterLevels', () => {
        const fetchParameters = {
            siteno: '413703087230501',
            agencyCd: 'USGS'
        };

        beforeEach(() => {
            fetch.resetMocks();
        });

        it('expects properly formated query parameters in service request', () => {
            fetchGroundwaterLevels(fetchParameters);

            expect(fetch.mock.calls).toHaveLength(1);
            const url = fetch.mock.calls[0][0].url;
            expect(url).toContain(`featureId=${fetchParameters.agencyCd}-${fetchParameters.siteno}`);
            expect(url).toContain('f=json');
        });

        it('Successful fetch returns a JSON object with ground water levels', async() => {
            fetch.once(MOCK_GW_FIELD_VISITS, {status: 200});
            const resp = await fetchGroundwaterLevels(fetchParameters);

            expect(resp).toEqual(JSON.parse(MOCK_GW_FIELD_VISITS));
        });

        it('Bad fetch returns an empty object', async() => {
            fetch.once('{}', {status: 400});
            const resp = await fetchGroundwaterLevels(fetchParameters);

            expect(resp).toEqual({});
        });

        it('handles a bad fetch', async() => {
            fetch.mockReject(new Error('fake error message'));
            const resp = await fetchGroundwaterLevels(fetchParameters);

            expect(resp).toEqual({});
        });
    });
});