import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';

import {MOCK_STATISTICS_RDB} from 'ui/mock-service-data';

import {getStatisticsServiceURL, fetchSiteStatistics} from './statistics-data';


describe('web-services/statistics-data', () => {
    let restoreConsole;

    beforeEach(() => {
        enableFetchMocks();
        restoreConsole = mockConsole();
    });

    afterEach(() => {
        disableFetchMocks();
        restoreConsole();
    });

    describe('getStatisticsServiceURL', () => {
        it('Expects the parameters specified will appear in the URL', () => {
            const result = getStatisticsServiceURL({
                siteno: '11112222',
                parameterCode: '72019',
                statType: 'max',
                format: 'rdb'
            });
            expect(result).toContain('sites=11112222');
            expect(result).toContain('statReportType=daily');
            expect(result).toContain('statTypeCd=max');
            expect(result).toContain('parameterCd=72019');
            expect(result).toContain('format=rdb');
        });
    });

    describe('fetchSiteStatistics', () => {
        const siteno = '05370000';
        const statType = 'median';
        const parameterCode = '00060';

        it('Gets a full year of statistical data', () => {
            fetchSiteStatistics({siteno, statType, parameterCode});
            const url = fetch.mock.calls[0][0];

            expect(url).toContain('statTypeCd=median');
            expect(url).toContain('parameterCd=00060');
            expect(url).toContain('sites=05370000');
        });

        it('Parses the data as expected', async() => {
            fetch.once(MOCK_STATISTICS_RDB, {status: 200});
            const resp = await fetchSiteStatistics({siteno, statType, parameterCode});

            expect(resp).toEqual({
                '00010': {
                    '153885': [{
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '1',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '16'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '2',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '16'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '3',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '16'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '4',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '5',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '6',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '7',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '8',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '9',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '10',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '11',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '12',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }, {
                        agency_cd: 'USGS',
                        site_no: '05370000',
                        parameter_cd: '00010',
                        ts_id: '153885',
                        loc_web_ds: '',
                        month_nu: '1',
                        day_nu: '13',
                        begin_yr: '1969',
                        end_yr: '2017',
                        count_nu: '49',
                        p50_va: '15'
                    }]
                }
            });
        });

        it('Returns an empty object on a failed request', async() => {
            fetch.once('Bad request', {status: 400});
            const resp = await fetchSiteStatistics({siteno, statType, parameterCode});

            expect(resp).toEqual({});
        });
    });
});
